﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.DataAnalysis.Benchmarks {
  public abstract class RegressionBenchmark : Benchmark, IRegressionBenchmarkProblemDataGenerator {

    protected RegressionBenchmark() { }
    protected RegressionBenchmark(RegressionBenchmark original, Cloner cloner)
      : base(original, cloner) {
    }

    #region properties
    public string TargetVariable { get; protected set; }
    public Dictionary<string, IntRange> Inputvariables { get; protected set; }
    public int AmountOfPoints { get; protected set; }
    public IntRange TrainingPartition { get; protected set; }
    public IntRange TestPartition { get; protected set; }
    #endregion

    protected abstract double CalculateFunction(Dictionary<string, IList<double>> data, List<string> vars);

    public IDataAnalysisProblemData GenerateProblemData() {
      //prepare dictionary
      Dictionary<string, IList<double>> data = new Dictionary<string, IList<double>>();
      data.Add(TargetVariable, new List<double>());
      foreach (var variable in Inputvariables.Keys) {
        data.Add(variable, new List<double>());
      }

      data = CalculateValues(data);

      List<IList> values = new List<IList>();
      foreach (var valueList in data.Values) {
        values.Add((IList)valueList);
      }

      Dataset dataset = new Dataset(data.Keys, values);
      dataset.Name = this.Name;

      RegressionProblemData problemData = new RegressionProblemData(dataset, dataset.DoubleVariables.Skip(1), dataset.DoubleVariables.First());

      #region set ProblemData specific parameters
      problemData.Name = "Data generated for benchmark problem \"" + this.Name + "\"";

      problemData.TargetVariableParameter.Value =
        problemData.TargetVariableParameter.ValidValues.First(v => v.Value == TargetVariable);
      problemData.InputVariables.SetItemCheckedState(
        problemData.InputVariables.Single(x => x.Value == TargetVariable), false);

      foreach (var variable in this.Inputvariables) {
        problemData.InputVariables.SetItemCheckedState(
          problemData.InputVariables.Single(x => x.Value == variable.Key), true);
      }

      problemData.TestPartition.Start = this.TestPartition.Start;
      problemData.TestPartition.End = this.TestPartition.End;
      problemData.TrainingPartition.Start = this.TrainingPartition.Start;
      problemData.TrainingPartition.End = this.TrainingPartition.End;
      #endregion

      return problemData;
    }

    private Dictionary<string, IList<double>> CalculateValues(Dictionary<string, IList<double>> data) {
      Random rand = new Random();
      List<string> vars = new List<string>(Inputvariables.Keys);
      for (int i = 0; i < AmountOfPoints; i++) {
        foreach (var variable in vars) {
          data[variable].Add(rand.NextDouble() *
            (Inputvariables[variable].End - Inputvariables[variable].Start) +
            Inputvariables[variable].Start);
        }

        data[TargetVariable].Add(CalculateFunction(data, vars));
      }
      return data;
    }
  }
}
